import "Turbine.UI";
import "Turbine.UI.Lotro";
import "Turbine.Gameplay";
import "Turbine.Utils";
import "Deusdictum.UI";
import "FalstarPlugins.Aura";

AuraWindow = class(Turbine.UI.Window);

function AuraWindow:Constructor(iwindow)

	Turbine.UI.Window.Constructor(self);
	
    self.left = data.windowTable[iwindow]:GetLeft();
	self.top = data.windowTable[iwindow]:GetTop();
	
--	Turbine.Shell.WriteLine(self.left .. " | " .. self.top);
	
	self.nrow = data.windowTable[iwindow]:GetNumberOfRows();
	self.ncol = data.windowTable[iwindow]:GetNumberOfColumns(); 

	self.aura_width = data.windowTable[iwindow]:GetAuraSize();
	self.font_size = (self.aura_width / 4) + 6; -- 16x16 has 10pt font, 64x64 has 22pt font
	self.aura_height = self.aura_width + self.font_size;

	self.padding = data.windowTable[iwindow]:GetPadding();
	self.back_opacity = data.windowTable[iwindow]:GetBackgroundOpacity();

    local width = self.ncol * (self.aura_width + self.padding) + self.padding;
    local height = self.nrow * (self.aura_height + self.padding) + self.padding;

--	Turbine.Shell.WriteLine("Aura Window Opacity: " .. data.back_opacity);
    local displayWidth, displayHeight = Turbine.UI.Display.GetSize();       
	self:SetPosition(math.round(self.left*displayWidth), math.round(self.top*displayHeight));
	self:SetVisible(true);
	self:SetBackColor(Turbine.UI.Color(self.back_opacity, 0, 0, 0));
	self:SetSize(width, height);
--	self:SetZOrder(100);

    self.player = Turbine.Gameplay.LocalPlayer.GetInstance();
    self.effects = self.player:GetEffects();

	self.DragBar = Deusdictum.UI.DragBar(self, data.windowTable[iwindow]:GetName());

    self.auraEffectList = {};

    local irow, icol, index;
    local left, top;
    local aura, auraEffect;
    for irow = 1, self.nrow do

		top = (irow-1)*(self.aura_height + self.padding) + self.padding;
    	for icol = 1, self.ncol do

    		left = (icol-1)*(self.aura_width + self.padding) + self.padding;

    		index = (irow-1)*self.ncol + icol;
    		aura = data.windowTable[iwindow]:FindAura(index);
    		
    		if (aura ~= nil) then

			    auraEffect = AuraEffect();
				auraEffect:SetParent(self);
				auraEffect:SetAuraSize(self.aura_width, self.font_size);
				auraEffect:SetWindowOpacity(self.back_opacity);
				auraEffect:SetFlashing(aura:GetFlashing());
			    auraEffect:SetPosition(left, top);
				auraEffect:SetAura(aura);
			    table.insert(self.auraEffectList, auraEffect);
		    
		    end

	    end

	end
	
	self:Update();

    self.effects.EffectAdded = function(sender, args)
        self:Update();
    end

    self.effects.EffectRemoved = function(sender, args)
    	self:Update();
--        self:RemoveEffect(args.Effect);
    end       
    
    -- add a context menu to show the options
	self.contextMenu = Turbine.UI.ContextMenu();
    self.menuItems = self.contextMenu:GetItems();
    self.menuItems:Add(Turbine.UI.MenuItem("Options"));
    self.menuItems:Add(Turbine.UI.MenuItem("Library"));

    self.menuItems:Get(1).Click = function(sender, args)
        ShowOptions();
    end;
    self.menuItems:Get(2).Click = function(sender, args)
        ShowLibrary();
    end;
    
    if (data.windowTable[iwindow]:GetShowContextMenu()) then
--    	Turbine.Shell.WriteLine("Show window");
    	self:SetMouseVisible(true);
    else                                                    
--    	Turbine.Shell.WriteLine("Do no show window");
    	self:SetMouseVisible(false);
    end

    self.MouseClick = function(sender, args)               
        if (args.Button == Turbine.UI.MouseButton.Right) then
            self.contextMenu:ShowMenu();
        end
    end
	
--	self.MouseClick = function(sender, args)               
--        if (args.Button == Turbine.UI.MouseButton.Right and
--			data.windowTable[iwindow]:GetShowContextMenu()) then
--            self.contextMenu:ShowMenu();
--        end
--    end     
	        
	self.Closing = function(sender, args)
--	    Turbine.Shell.WriteLine("Aura window closing");
	    self.DragBar = nil;
	end

end


function AuraWindow:Update()

	local ii;
	for ii = 1, table.getn(self.auraEffectList) do
		
		local aura = self.auraEffectList[ii]:GetAura();
		if (aura ~= nil) then
		                        
			local effect = self:GetEffect(aura);
			if (self:ShowAura(aura, effect)) then
			
			    if (effect ~= nil) then
					self.auraEffectList[ii]:SetEffect(effect);
				end

				self.auraEffectList[ii]:SetVisible(true);
			
			else
			
				self.auraEffectList[ii]:SetVisible(false);

			end
			
			
--			if (effect == nil) then
--			
--				if (aura:GetShowActive()) then    
--					self.auraEffectList[ii]:SetVisible(false);
--				else
--				end
--			
--			else
--			
--				if (aura:GetShowActive()) then				
--					self.auraEffectList[ii]:SetEffect(effect);
--					self.auraEffectList[ii]:SetVisible(true);	
--				else
--					self.auraEffectList[ii]:SetVisible(false);
--				end
--							
--			end
		
		end
	
	end

end

function AuraWindow:ShowAura(aura, effect)

 	local active = false;
	local category = false;
	
	if (effect == nil) then
	
		active = not aura:GetShowActive();
		category = true;
		
	else
	
	    active =     aura:GetShowActive();
	    if (Turbine.Gameplay.EffectCategory.Undefined ~= aura:GetCategory()) then
		    category = aura:GetCategory() == effect:GetCategory();
		else
			category = true;
		end
	    
	end
	
	return (active and category);

end

function AuraWindow:RemoveEffect(effect)

--	Turbine.Shell.WriteLine("RemoveEffect " .. effect:GetID());

	local ii;
	local effect_id = effect:GetID();
	for ii = 1, table.getn(self.auraEffectList) do

		if (effect_id == self.auraEffectList[ii]:GetID()) then
--			if self.auraEffectList[ii]:GetShowActive() then
				self.auraEffectList[ii]:SetVisible(false);
--			else
--			    self.auraEffectList[ii]:SetVisible(true);
--			end
  		end
		
	end

end

function AuraWindow:GetEffect(aura)

	local effect = nil;
	local ieffect, jeffect, ipattern;
	local neffect = self.effects:GetCount();
	local curable_flag;
	
	for ieffect = 1, neffect do
	
		jeffect = neffect + 1 - ieffect;   
	    effect = self.effects:Get(jeffect);
	    
	    curable_flag = true;
	    if (aura:GetCurable() and not effect:IsCurable()) then
	    	curable_flag = false;
	    end
	    	    
	    if (curable_flag) then
	    
		    if ((Turbine.Gameplay.EffectCategory.Undefined == aura:GetCategory()) or
				(effect:GetCategory() == aura:GetCategory())) then
		    
				if (aura:GetNumberOfPatterns() == 0) then      	    
					return effect;
				else
				
					for ipattern = 1, aura:GetNumberOfPatterns() do	
						if (aura:GetExactMatch()) then
							if (effect:GetName() == aura:GetPatternText(ipattern)) then
		--			        	Turbine.Shell.WriteLine("GetEffect: " .. effect:GetName());
							    return effect;
							end
						else                          
							if (string.find(effect:GetName(), aura:GetPatternText(ipattern)) ~= nil) then
		--			        	Turbine.Shell.WriteLine("GetEffect: " .. effect:GetName());
							    return effect;
							end
						end
					end
	
			    end
			    
		    end
		    
	    end

    end		
	
--	if (aura:GetNumberOfPatterns() == 0) then
--	
----	    Turbine.Shell.WriteLine("GetAura no pattern: " .. aura:GetName());
--
--	    if (Turbine.Gameplay.EffectCategory.Undefined ~= aura:GetCategory()) then
--
--			for ieffect = 1, neffect do
--			    jeffect = neffect + 1 - ieffect;
--			    effect = self.effects:Get(jeffect);
--			    if (effect:GetCategory() == aura:GetCategory()) then
----			        Turbine.Shell.WriteLine("GetEffect: " .. effect:GetName());
--					return effect;
--			    end
--			end
--		
--		end
--		
--	else
--	
--		for ipattern = 1, aura:GetNumberOfPatterns() do
--	--		Turbine.Shell.WriteLine("GetEffect: " .. aura:GetPatternText(ipattern));
--		    for ieffect = 1, neffect do
--
--				-- want to get the latest effect if multiple exist and
--				-- new effects are added to the end of the effect list
--				jeffect = neffect + 1 - ieffect;
--
--				effect = self.effects:Get(jeffect);				
--				if (aura:GetExactMatch()) then
--					if (effect:GetName() == aura:GetPatternText(ipattern)) then
----			        	Turbine.Shell.WriteLine("GetEffect: " .. effect:GetName());
--					    return effect;
--					end
--				else                          
--					if (string.find(effect:GetName(), aura:GetPatternText(ipattern)) ~= nil) then
----			        	Turbine.Shell.WriteLine("GetEffect: " .. effect:GetName());
--					    return effect;
--					end
--				end
--
--		    end
--	    end
--
--    end
    
    return nil;

end