import "Turbine.Utils";
import "FalstarPlugins.Aura";

Settings = class();

function Settings:Constructor()
                      
    self:SetOptions();
    self:SetLibrary();

    local saved = Turbine.PluginData.Load(Turbine.DataScope.Character, "AuraSettings");
                                         	
    if (type(saved) ~= "table") then
    
        -- save the default options
        self:Save();
        
    else
    
        if (saved.left ~= nil) then
			self.left = saved.left;
			if (self.left < 0.0 or self.left > 1.0) then
				self.left = 0.5;
			end
--			Turbine.Shell.WriteLine("Load left: " .. self.left);
		end
		
		if (saved.top ~= nil) then
        	self.top = saved.top; 
			if (self.top < 0.0 or self.top > 1.0) then
				self.top = 0.5;
			end
--			Turbine.Shell.WriteLine("Load top: " .. self.top);
        end

        if (saved.options_left ~= nil) then
            self.options_left = saved.options_left; 
			if (self.options_left < 0.0 or self.options_left > 1.0) then
				self.options_left = 0.2;
			end
--			Turbine.Shell.WriteLine("Load options left: " .. self.options_left);
        end

        if (saved.options_top ~= nil) then
            self.options_top = saved.options_top;   
			if (self.options_top < 0.0 or self.options_top > 1.0) then
				self.options_top = 0.1;
			end
--			Turbine.Shell.WriteLine("Load options top: " .. self.options_top);
        end

        if (saved.library_left ~= nil ) then
            self.library_left = saved.library_left;     
			if (self.library_left < 0.0 or self.library_left > 1.0) then
				self.library_left = 0.5;
			end
--			Turbine.Shell.WriteLine("Load library left: " .. self.library_left);
        end

        if (saved.library_top ~= nil ) then
            self.library_top = saved.library_top;      
			if (self.library_top < 0.0 or self.library_top > 1.0) then
				self.library_top = 0.1;
			end
--			Turbine.Shell.WriteLine("Load library top: " .. self.library_top);
        end

		if (saved.nrow ~= nil) then
        	self.nrow = saved.nrow;
--			Turbine.Shell.WriteLine("Load nrow: " .. self.nrow);
        end

		if (saved.ncol ~= nil) then
        	self.ncol = saved.ncol;
--			Turbine.Shell.WriteLine("Load ncol: " .. self.ncol);
        end

		if (saved.padding ~= nil) then
        	self.padding = saved.padding;
--			Turbine.Shell.WriteLine("Load padding: " .. self.padding);
        end

		if (saved.aura_width ~= nil) then
        	self.aura_width = saved.aura_width;
--			Turbine.Shell.WriteLine("Load aura_width: " .. self.aura_width);
        end

		if (saved.aura_height ~= nil) then
        	self.aura_height = saved.aura_height;
--			Turbine.Shell.WriteLine("Load aura_height: " .. self.aura_height);
        end
        
        if (saved.back_opacity ~= nil) then
        	self.back_opacity = saved.back_opacity;
--        	Turbine.Shell.WriteLine("Load back_opacity: " .. self.back_opacity);
        end
        
        if (saved.aura_index ~= nil) then
        	self.aura_index = saved.aura_index;
--        	Turbine.Shell.WriteLine("Load aura_index: " .. self.aura_index);
        end

        if (saved.window_index ~= nil) then
        	self.window_index = saved.window_index;
--        	Turbine.Shell.WriteLine("Load window_index: " .. self.window_index);
        end
        
        if (saved.auraTable ~= nil) then
            self.auraTable = {};
            local ii, aura;
            for ii = 1, table.getn(saved.auraTable) do
                aura = Aura();
                aura:SetName(saved.auraTable[ii].name);
                aura:SetID(saved.auraTable[ii].id);
				aura:SetPattern(saved.auraTable[ii].pattern);
				aura:SetShowTimer(saved.auraTable[ii].showTimer);
                aura:SetShowActive(saved.auraTable[ii].showActive);
                aura:SetExactMatch(saved.auraTable[ii].exactMatch);
                aura:SetFlashing(saved.auraTable[ii].flashing);
                aura:SetCurable(saved.auraTable[ii].curable);
                aura:SetBackground(saved.auraTable[ii].background);
                aura:SetCategory(saved.auraTable[ii].my_cat);
				self.auraTable[ii] = aura;
--				Turbine.Shell.WriteLine("Load auraTable(" .. ii .. "): " .. aura:GetName());
			end
        end
        
        if (saved.windowTable ~= nil) then
            self.windowTable = {};
            local ii, window;
            for ii = 1 , table.getn(saved.windowTable) do
                window = WindowSettings();
                window:SetName(saved.windowTable[ii].name);
                window:SetLeft(saved.windowTable[ii].left);
                window:SetTop(saved.windowTable[ii].top);
                window:SetNumberOfRows(saved.windowTable[ii].nrow);
                window:SetNumberOfColumns(saved.windowTable[ii].ncol);
                window:SetPadding(saved.windowTable[ii].padding);
                window:SetAuraSize(saved.windowTable[ii].size);
                window:SetBackgroundOpacity(saved.windowTable[ii].opacity);
                window:SetAuraMap(saved.windowTable[ii].auraMap);
                window:SetShowContextMenu(saved.windowTable[ii].menu);
                window:SetShowWindow(saved.windowTable[ii].visible);
                self.windowTable[ii] = window;
            end
        end
        
    end
    
end

--function Settings:GetNewHash()
--
--	local hash = math.round(math.random()*1.0e8);
--	
--	local ii;
--	for ii = 1, table.getn(data.auraMap) do
--		if (data.auraMap[ii]:GetID() == hash) then
--			hash = self:GetNewHash();
--			break;
--		end
--	end
--	
--	return hash;
--	
--end

function Settings:SavePosition()

	local displayWidth, displayHeight = Turbine.UI.Display.GetSize();   
	
	local ii;
	if (auraWindow ~= nil) then
		for ii = 1, table.getn(auraWindow) do
			if (auraWindow[ii] ~= nil) then
				if (self.windowTable[ii] ~= nil) then
					self.windowTable[ii]:SetLeft(auraWindow[ii]:GetLeft() / displayWidth);
					self.windowTable[ii]:SetTop(auraWindow[ii]:GetTop() / displayHeight);    
				end
			end
		end
	end

	if (optionsWindow ~= nil) then
		self.options_left = optionsWindow:GetLeft() / displayWidth;
		self.options_top = optionsWindow:GetTop() / displayHeight;
	end

	if (libraryWindow ~= nil) then
		self.library_left = libraryWindow:GetLeft() / displayWidth;
		self.library_top = libraryWindow:GetTop() / displayHeight;
	end

end

function Settings:Save()

--	Turbine.Shell.WriteLine("Save");    
	
	self:SavePosition();

	local ii, nn, item;
	local libraryCombo;
	if (libraryWindow ~= nil) then
	    if (libraryWindow.libraryPanel ~= nil) then
			libraryCombo = libraryWindow.libraryPanel.auraCombo;
		end
	end
	if (libraryCombo ~= nil) then

--		Turbine.Shell.WriteLine("Save auraCombo");

	    for ii = 1, table.getn(self.auraTable) do
	        aura = self.auraTable[ii]; aura = nil;
		end
		self.auraTable = {};

		for ii = 1, libraryCombo:GetItemCount() do
        	self.auraTable[ii] = libraryCombo:GetItem(ii);
--        	Turbine.Shell.WriteLine("aura[" .. ii .. "]: " .. self.auraTable[ii]:GetID());
		end
		
	end

	local optionsCombo;
 	if (optionsWindow ~= nil) then
 	    if (optionsWindow.optionsPanel ~= nil) then
 	        optionsCombo = optionsWindow.optionsPanel.windowCombo;
 	    end
 	end
 	if (optionsCombo ~= nil) then
 	
 	    for ii = 1, table.getn(self.windowTable) do
			window = self.windowTable[ii]; window = nil;
 	    end
 	    self.windowTable = {};
 	    
 	    for ii = 1, optionsCombo:GetItemCount() do
 	        self.windowTable[ii] = optionsCombo:GetItem(ii);
-- 	        Turbine.Shell.WriteLine("Saving window: " .. self.windowTable[ii]:GetName());
 	    end

	end
	
--	Turbine.Shell.WriteLine("Saving " .. table.getn(self.windowTable) .. " windows");
	
    Turbine.PluginData.Save(Turbine.DataScope.Character, "AuraSettings", self);
    
end

--function Settings:FindIndex(aura)
--
--	local ret = nil;
--	
--	local ii;
--	for ii = 1, table.getn(self.auraTable) do
--		if (self.auraTable[ii] == aura) then
--		    ret = ii;
--      		break;
--		end
--	end
--	
--	return ret;
--
--end

function Settings:SetOptions()
    
	self.version = 1.0;
	
	self.options_left = 0.2;
	self.options_top = 0.1;
	
	self.library_left = 0.5;
	self.library_top = 0.1;         	

	self.window_index = 1;
	
	local window = WindowSettings();
	window:SetName("Aura");

	window:SetLeft(0.5);
	window:SetTop(0.5);        

	window:SetNumberOfRows(1);
	window:SetNumberOfColumns(6);
	window:SetPadding(5);
	window:SetAuraSize(24);
	window:SetBackgroundOpacity(0.5);
	window:SetShowContextMenu(true);
	window:SetShowWindow(true);

	window:SetAuraMap({180, 181, 182, 183, 184, 185});
	
	self.windowTable = {};
	self.windowTable[1] = window;
	
end
	
function Settings:SetLibrary()
	
	self.aura_index = 1;
	                    	
	self.auraTable = {};
	
	local aura, id;
	local index = 1;
	
	-------------------------------- burglar -----------------------------------

	id = 100;
	aura = Aura();
	aura:SetName("BRG - Aim");
	aura:SetPattern({"Aiming"});
	aura:SetShowTimer(true);
	aura:SetID(id);
	self.auraTable[index] = aura;
	index = index + 1; id = id + 1;

	aura = Aura();
	aura:SetName("BRG - Critical Response");
	aura:SetPattern({"Critical Response"});
	aura:SetShowTimer(true);
	aura:SetID(id);
	self.auraTable[index] = aura;
	index = index + 1; id = id + 1;

	aura = Aura();
	aura:SetName("BRG - Feint");
	aura:SetPattern({"Feint"});
	aura:SetShowTimer(true);
	aura:SetID(id);
	self.auraTable[index] = aura;
	index = index + 1; id = id + 1;

	aura = Aura();
	aura:SetName("BRG - Quiet Knife");
	aura:SetPattern({"The Quiet Knife"});
	aura:SetShowTimer(true);
	aura:SetID(id);
	self.auraTable[index] = aura;
	index = index + 1; id = id + 1;

	aura = Aura();
	aura:SetName("BRG - Mischief Maker");
	aura:SetPattern({"The Mischief"});
	aura:SetShowTimer(true);
	aura:SetID(id);
	self.auraTable[index] = aura;
	index = index + 1; id = id + 1;

	aura = Aura();
	aura:SetName("BRG - The Gambler");
	aura:SetPattern({"The Gambler"});
	aura:SetShowTimer(true);
	aura:SetID(id);
	self.auraTable[index] = aura;
	index = index + 1; id = id + 1;
	
	-------------------------------- captain -----------------------------------          
	  
    id = 110;          	
	aura = Aura();
	aura:SetName("CPT - Parry Buff");
	aura:SetPattern({"On Guard"});
	aura:SetID(id);
	self.auraTable[index] = aura;
	index = index + 1; id = id + 1;

	aura = Aura();
	aura:SetName("CPT - War Cry");
	aura:SetPattern({"cry"});
	aura:SetShowTimer(true);
	aura:SetID(id);
	self.auraTable[index] = aura;
	index = index + 1; id = id + 1;

	aura = Aura();
	aura:SetName("CPT - Rallying Cry");
	aura:SetPattern({"Cry of Victory: Morale"});
	aura:SetShowTimer(true);
	aura:SetID(id);
	self.auraTable[index] = aura;
	index = index + 1; id = id + 1;

	aura = Aura();
	aura:SetName("CPT - Make Haste");
	aura:SetPattern({"Make Haste"});
	aura:SetShowTimer(true);
	aura:SetID(id);
	self.auraTable[index] = aura;
	index = index + 1; id = id + 1;

	aura = Aura();
	aura:SetName("CPT - Banner");
	aura:SetPattern({"Banner"});
	aura:SetShowActive(false);
	aura:SetBackground(6);
	aura:SetID(id);
	self.auraTable[index] = aura;
	index = index + 1; id = id + 1;
	
	-------------------------------- champion ----------------------------------          
	   
    id = 120;          	
	aura = Aura();
	aura:SetName("CHM - Flurry");
	aura:SetPattern({"Flurry"});
	aura:SetShowTimer(true); 
	aura:SetID(id);
	self.auraTable[index] = aura;
	index = index + 1; id = id + 1;

	aura = Aura();
	aura:SetName("CHM - Controlled Burn");
	aura:SetPattern({"Controlled Burn"});
	aura:SetShowTimer(true);
	aura:SetID(id);
	self.auraTable[index] = aura;
	index = index + 1; id = id + 1;

	aura = Aura();
	aura:SetName("CHM - Stance");
	aura:SetPattern({"Fervour", "Ardour", "Glory"});
	aura:SetShowActive(false);
	aura:SetBackground(5);
	aura:SetID(id);
	self.auraTable[index] = aura;
	index = index + 1; id = id + 1;
	
	-------------------------------- guardian ----------------------------------      
	          
    id = 130;          	
	aura = Aura();
	aura:SetName("GRD - Guardian's Ward");
	aura:SetPattern({"Guardian's Ward"});
	aura:SetShowTimer(true); 
	aura:SetID(id);
	self.auraTable[index] = aura;
	index = index + 1; id = id + 1;   
	
	aura = Aura();
	aura:SetName("GRD - Guardian's Pledge");
	aura:SetPattern({"Guardian's Pledge"});
	aura:SetShowTimer(true); 
	aura:SetID(id);
	self.auraTable[index] = aura;
	index = index + 1; id = id + 1;  
	
	aura = Aura();
	aura:SetName("GRD - Stance");
	aura:SetPattern({"Guardian's Block Stance", "Guardian's Parry Stance", "Overpower"}); 
	aura:SetShowActive(false);
	aura:SetBackground(1);
	aura:SetID(id);
	self.auraTable[index] = aura;
	index = index + 1; id = id + 1;
	
	--------------------------------- hunter -----------------------------------        
	     
    id = 140;
	aura = Aura();
	aura:SetName("HNT - Swift Stroke");
	aura:SetPattern({"Swift Stroke"});
	aura:SetShowTimer(true);
	aura:SetID(id);
	self.auraTable[index] = aura;
	index = index + 1; id = id + 1;

	aura = Aura();
	aura:SetName("HNT - Burn Hot");
	aura:SetPattern({"Burn Hot"});
	aura:SetShowTimer(true);
	aura:SetID(id);
	self.auraTable[index] = aura;
	index = index + 1; id = id + 1;

	aura = Aura();
	aura:SetName("HNT - Cool Burn");
	aura:SetPattern({"Cool Burn"});
	aura:SetShowTimer(true);
	aura:SetID(id);
	self.auraTable[index] = aura;
	index = index + 1; id = id + 1;

	aura = Aura();
	aura:SetName("HNT - Find the Path");
	aura:SetPattern({"Find the Path"});
	aura:SetShowTimer(true);
	aura:SetID(id);
	self.auraTable[index] = aura;
	index = index + 1; id = id + 1;

	aura = Aura();
	aura:SetName("HNT - Stance");
	aura:SetPattern({"Stance: Precision", "Stance: Strength", "Stance: Endurance"});
	aura:SetShowActive(false);
	aura:SetBackground(2);
	aura:SetID(id);
	self.auraTable[index] = aura;
	index = index + 1; id = id + 1;
	
	------------------------------- loremaster ---------------------------------      
	         
    id = 150;
	aura = Aura();
	aura:SetName("LRM - Vigilance");
	aura:SetPattern({"Sign of Power: Vigilance"});
	aura:SetShowTimer(true);
	aura:SetID(id);
	self.auraTable[index] = aura;
	index = index + 1; id = id + 1;

	aura = Aura();
	aura:SetName("LRM - Righteousness");
	aura:SetPattern({"Sign of Power: Righteousness"});
	aura:SetShowTimer(true);
	aura:SetID(id);
	self.auraTable[index] = aura;
	index = index + 1; id = id + 1;

	aura = Aura();
	aura:SetName("LRM - Ancient Wisdom");
	aura:SetPattern({"Ancient Wisdom"});
	aura:SetShowActive(false);
	aura:SetBackground(7);
	aura:SetID(id);
	self.auraTable[index] = aura;
	index = index + 1; id = id + 1;

	aura = Aura();
	aura:SetName("LRM - Wisdom of the Council");
	aura:SetPattern({"Wisdom of the Council"});
	aura:SetShowTimer(true);
	aura:SetID(id);
	self.auraTable[index] = aura;
	index = index + 1; id = id + 1;
	
	-------------------------------- minstrel ---------------------------------- 

	id = 160;          	
	aura = Aura();
	aura:SetName("MIN - Soliloquy of Spirit");
	aura:SetPattern({"Soliloquy of Spirit"});
	aura:SetShowTimer(true); 
	aura:SetID(id);
	self.auraTable[index] = aura;
	index = index + 1; id = id + 1;
	
	aura = Aura();
	aura:SetName("MIN - Tales");
	aura:SetPattern({"Tale of Heroism", "Tale of Battle", "Tale of Warding", "Tale of Frost and Flame"}); 
	aura:SetShowActive(false);
	aura:SetBackground(10);
	aura:SetID(id);
	self.auraTable[index] = aura;
	index = index + 1; id = id + 1;
	
	aura = Aura();
	aura:SetName("MIN - Ballad of Resonance");
	aura:SetPattern({"Ballad of Resonance"});
	aura:SetShowTimer(true); 
	aura:SetID(id);
	self.auraTable[index] = aura;
	index = index + 1; id = id + 1;
	
	aura = Aura();
	aura:SetName("MIN - Ballad of War");
	aura:SetPattern({"Ballad of War"});
	aura:SetShowTimer(true); 
	aura:SetID(id);
	self.auraTable[index] = aura;
	index = index + 1; id = id + 1;
	
	aura = Aura();
	aura:SetName("MIN - Anthem of the Valar");
	aura:SetPattern({"Anthem of the Valar"});
	aura:SetShowTimer(true); 
	aura:SetID(id);
	self.auraTable[index] = aura;
	index = index + 1; id = id + 1;        
	
	aura = Aura();
	aura:SetName("MIN - Cry of the Chorus");
	aura:SetPattern({"Cry of The Chorus Silence Immunity"});
	aura:SetShowTimer(true); 
	aura:SetID(id);
	self.auraTable[index] = aura;
	index = index + 1; id = id + 1;
	
	------------------------------- rune keeper --------------------------------

	id = 170;
	aura = Aura();
	aura:SetName("RNK - Prelude to Hope");
	aura:SetPattern({"Prelude to Hope"});
	aura:SetShowTimer(true); 
	aura:SetID(id);
	self.auraTable[index] = aura;
	index = index + 1; id = id + 1;

	aura = Aura();
	aura:SetName("RNK - Mending Verse");
	aura:SetPattern({"Mending Verse"});
	aura:SetShowTimer(true);  
	aura:SetID(id);
	self.auraTable[index] = aura;
	index = index + 1; id = id + 1;

	aura = Aura();
	aura:SetName("RNK - Thunderous Words");
	aura:SetPattern({"Thunderous Words"});
	aura:SetShowTimer(true);  
	aura:SetID(id);
	self.auraTable[index] = aura;
	index = index + 1; id = id + 1;

	aura = Aura();
	aura:SetName("RNK - Harsh Debate");
	aura:SetPattern({"Harsh Debate"});
	aura:SetShowTimer(true);  
	aura:SetID(id);
	self.auraTable[index] = aura;
	index = index + 1; id = id + 1;

	aura = Aura();
	aura:SetName("RNK - Charged Runes");
	aura:SetPattern({"Charged Runes"});
	aura:SetShowTimer(true);
	aura:SetID(id);
	self.auraTable[index] = aura;
	index = index + 1; id = id + 1;

	aura = Aura();
	aura:SetName("RNK - Perfect Imagery");
	aura:SetPattern({"Perfect Imagery"});
	aura:SetShowTimer(true);
	aura:SetID(id);
	self.auraTable[index] = aura;
	index = index + 1; id = id + 1;
	
	--------------------------------- warden -----------------------------------

	id = 180;
	
	aura = Aura();
	aura:SetName("WRD - Shield Mastery");
	aura:SetPattern({"Shield Mastery"});
	aura:SetShowTimer(true);
	aura:SetID(id);
	self.auraTable[index] = aura;
	index = index + 1; id = id + 1;

	aura = Aura();
	aura:SetName("WRD - Wall of Steel");
	aura:SetPattern({"Wall of Steel"});
	aura:SetShowTimer(true);
	aura:SetID(id);
	self.auraTable[index] = aura;
	index = index + 1; id = id + 1;

	aura = Aura();
	aura:SetName("WRD - Dance of War");
	aura:SetPattern({"Dance of Battle"});
	aura:SetShowTimer(true);
	aura:SetID(id);
	self.auraTable[index] = aura;
	index = index + 1; id = id + 1;

	aura = Aura();
	aura:SetName("WRD - Defiant Challenge");
	aura:SetPattern({"Defiant Challenge"});
	aura:SetShowTimer(true);
	aura:SetID(id);
	self.auraTable[index] = aura;
	index = index + 1; id = id + 1;

	aura = Aura();
	aura:SetName("WRD - Warden's Triumph");
	aura:SetPattern({"Warden's Triumph"});
	aura:SetShowTimer(true);
	aura:SetID(id);
	self.auraTable[index] = aura;
	index = index + 1; id = id + 1;

	aura = Aura();
	aura:SetName("WRD - Stance");
	aura:SetPattern({"Recklessness", "Conservation", "Determination"});
	aura:SetShowActive(false);
	aura:SetBackground(9);
	aura:SetID(id);
	self.auraTable[index] = aura;
	index = index + 1; id = id + 1;

	aura = Aura();
	aura:SetName("WRD - Conviction");
	aura:SetPattern({"Conviction"});
	aura:SetShowTimer(true);
	aura:SetID(id);
	self.auraTable[index] = aura;
	index = index + 1; id = id + 1;

	aura = Aura();
	aura:SetName("WRD - Restoration");
	aura:SetPattern({"Tier 4 Heal over Time"});
	aura:SetShowTimer(true);
	aura:SetID(id);
	self.auraTable[index] = aura;
	index = index + 1; id = id + 1;
	
	--------------------------------- debuff -----------------------------------

	id = 200;

	aura = Aura();
	aura:SetName("Any Wound");
	aura:SetPattern({""});
	aura:SetCategory(Turbine.Gameplay.EffectCategory.Wound);
	aura:SetShowTimer(true);
	aura:SetID(id);
	self.auraTable[index] = aura;
	index = index + 1; id = id + 1;

	aura = Aura();
	aura:SetName("Any Fear");
	aura:SetPattern({""});
	aura:SetCategory(Turbine.Gameplay.EffectCategory.Fear);
	aura:SetShowTimer(true);
	aura:SetID(id);
	self.auraTable[index] = aura;
	index = index + 1; id = id + 1;

	aura = Aura();
	aura:SetName("Any Poison");
	aura:SetPattern({""});
	aura:SetCategory(Turbine.Gameplay.EffectCategory.Poison);
	aura:SetShowTimer(true);
	aura:SetID(id);
	self.auraTable[index] = aura;
	index = index + 1; id = id + 1;

	aura = Aura();
	aura:SetName("Any Disease");
	aura:SetPattern({""});
	aura:SetCategory(Turbine.Gameplay.EffectCategory.Disease);
	aura:SetShowTimer(true);
	aura:SetID(id);
	self.auraTable[index] = aura;
	index = index + 1; id = id + 1;
    
end
